import 'dart:async';

import 'package:notice/filters.dart';
import 'package:notice/notice.dart';
import 'package:notice/outputs.dart';
import 'package:notice/version.dart';
import 'package:notice_sentry/src/conversions.dart';
import 'package:sentry/sentry.dart';

const _defaultMinBreadcrumbLevel = NoticeLevel.info;
const _defaultMinEventLevel = NoticeLevel.error;

/// An [Integration] which listens to all messages of the
/// [logging](https://pub.dev/packages/logging) package.
class NoticeIntegration implements Integration<SentryOptions>, NoticeOutput {
  /// Creates the [NoticeIntegration].
  ///
  /// All log events equal or higher than [minBreadcrumbLevel] or
  /// filtered by [breadcrumbFilter] are recorded as a [Breadcrumb].
  /// All log events equal or higher than [minEventLevel] or
  /// filtered by [eventFilter] are recorded as a [SentryEvent].
  NoticeIntegration({
    NoticeLevel? minBreadcrumbLevel,
    NoticeLevel? minEventLevel,
    NoticeFilter? breadcrumbFilter,
    NoticeFilter? eventFilter,
  })  : assert(minBreadcrumbLevel == null || breadcrumbFilter == null,
            'Only one of minBreadcrumbLevel or breadcrumbFilter can be set.'),
        assert(minEventLevel == null || eventFilter == null,
            'Only one of minEventLevel or eventFilter can be set.'),
        _breadcrumbFilter = breadcrumbFilter ??
            LevelFilter(minBreadcrumbLevel ?? _defaultMinBreadcrumbLevel),
        _eventFilter =
            eventFilter ?? LevelFilter(minEventLevel ?? _defaultMinEventLevel);

  final NoticeFilter _breadcrumbFilter;
  final NoticeFilter _eventFilter;
  late Hub _hub;

  @override
  void call(Hub hub, SentryOptions options) {
    _hub = hub;
    options.sdk.addPackage("notice", version);
    options.sdk.addIntegration('LoggingIntegration');
  }

  @override
  Future<void> close() async {}

  Future<void> _dispatch(NoticeRecord record) async {
    // The event must be logged first, otherwise the log would also be added
    // to the breadcrumbs for itself.
    if (_eventFilter.filter(record)) {
      await _hub.captureEvent(
        record.toEvent(),
        stackTrace: record.stackTrace,
        hint: Hint.withMap({TypeCheckHint.record: record}),
      );
    }

    if (_breadcrumbFilter.filter(record)) {
      await _hub.addBreadcrumb(
        record.toBreadcrumb(),
        hint: Hint.withMap({TypeCheckHint.record: record}),
      );
    }
  }

  @override
  void onRecord(NoticeRecord record) {
    unawaited(_dispatch(record));
  }
}
